

///////////////////////////////////////////////////////////
// functions

//------------------------------------------------
// screen
#define SCREEN_WIDTH				(720.0)
#define SCREEN_HEIGHT				(480.0)
#define SCREEN_ASPECT				(SCREEN_WIDTH /SCREEN_HEIGHT)
#define SCREEN_FOVY_DEG				(45.0)
#define SCREEN_FOVY_RAD				radians(SCREEN_FOVY_DEG)
#define SCREEN_FOVY_TAN				tan( SCREEN_FOVY_RAD )
#define SCREEN_HALF_FOVY_TAN		tan( SCREEN_FOVY_RAD *0.5 )

#define LUMA_COEFF					vec3(0.299, 0.587, 0.114)

float calcLuma( in vec3 color )
{
	return dot( color, LUMA_COEFF );
}


//------------------------------------------------
// compression
#define CHANNEL_RANGE_MAX			(255.0)
#define CHANNEL_RANGE_SUPERIOR		(256.0)

#if 0
vec2 compress16( in float value )
{
	float revised = value * CHANNEL_RANGE_MAX / CHANNEL_RANGE_SUPERIOR;
	return fract( vec2( revised, fract(revised *CHANNEL_RANGE_MAX) ) );
}

float decompress16( in vec2 value )
{
	float decomp = dot( value, vec2( 1.0, 1.0 /CHANNEL_RANGE_MAX ) );
	float revised = decomp * CHANNEL_RANGE_SUPERIOR / CHANNEL_RANGE_MAX;
	return revised;
}

vec4 compress32( in float value )
{
	float revised = value * CHANNEL_RANGE_MAX / CHANNEL_RANGE_SUPERIOR;
	return fract( vec4( revised, fract(revised *CHANNEL_RANGE_MAX), fract(revised *CHANNEL_RANGE_MAX *CHANNEL_RANGE_MAX), fract(revised *CHANNEL_RANGE_MAX *CHANNEL_RANGE_MAX *CHANNEL_RANGE_MAX) ) );
}

float decompress32( in vec4 value )
{
	float decomp = dot( value, vec4( 1.0, 1.0 /CHANNEL_RANGE_MAX, 1.0 /CHANNEL_RANGE_MAX /CHANNEL_RANGE_MAX, 1.0 /CHANNEL_RANGE_MAX /CHANNEL_RANGE_MAX /CHANNEL_RANGE_MAX ) );
	float revised = decomp * CHANNEL_RANGE_SUPERIOR / CHANNEL_RANGE_MAX;
	return revised;
}
#else
vec2 compress16( in float value )
{
	vec2 comp = fract( vec2( 1.0, CHANNEL_RANGE_MAX ) *value );
	comp -= comp.yy * vec2( 1.0 /CHANNEL_RANGE_MAX, 0.0 );
	return comp;
}

float decompress16( in vec2 value )
{
	float decomp = dot( value, vec2( 1.0, 1.0 /CHANNEL_RANGE_MAX ) );
	return decomp;
}

vec4 compress32( in float value )
{
	vec4 comp = fract( vec4( 1.0, CHANNEL_RANGE_MAX, CHANNEL_RANGE_MAX*CHANNEL_RANGE_MAX, CHANNEL_RANGE_MAX*CHANNEL_RANGE_MAX*CHANNEL_RANGE_MAX ) *value );
	comp -= comp.yzww * vec4( 1.0 /CHANNEL_RANGE_MAX, 1.0 /CHANNEL_RANGE_MAX, 1.0 /CHANNEL_RANGE_MAX, 0.0 );
	return comp;
}

float decompress32( in vec4 value )
{
	float decomp = dot( value, vec4( 1.0, 1.0 /CHANNEL_RANGE_MAX, 1.0 /CHANNEL_RANGE_MAX /CHANNEL_RANGE_MAX, 1.0 /CHANNEL_RANGE_MAX /CHANNEL_RANGE_MAX /CHANNEL_RANGE_MAX ) );
	return decomp;
}
#endif

//------------------------------------------------
// depth
#define DEPTH_NEAR					(1.0)
#define DEPTH_FAR					(100.0)
#define DEPTH_RANGE					(DEPTH_FAR - DEPTH_NEAR)

float depth2nld( in float depth )
{
	return ( depth -DEPTH_NEAR ) /DEPTH_RANGE;
}

float nld2depth( in float nld )
{
	return nld *DEPTH_RANGE + DEPTH_NEAR;
}

//------------------------------------------------
// light
#if 0
#define IS_LIGHT_DIR_VIEW			( 0 )
#define LIGHT_DIR					vec3(-0.5774, -0.5774, -0.5774)
#else
#define IS_LIGHT_DIR_VIEW			( 1 )
#define LIGHT_DIR					(u_directionalLightDirection[0])// already in view space
#endif


//------------------------------------------------
// shadow
#define SHADOW_DEPTH_NEAR			(-100.0)
#define SHADOW_DEPTH_FAR			(100.0)
#define SHADOW_DEPTH_RANGE			(SHADOW_DEPTH_FAR - SHADOW_DEPTH_NEAR)

#define SHADOW_PROJ_WORLD_RANGE		(20.0)

#if 1
	// normalize(-1,-1,-1) YUp
#define SHADOW_DIR_X				vec3( 0.7071,  0.0000,  0.7071)
#define SHADOW_DIR_Y				vec3(-0.4082,  0.8165, -0.4082)
#define SHADOW_DIR_Z				vec3(-0.5774, -0.5774, -0.5774)
#define SHADOW_ORIGIN				vec3(0.0, 0.0, 0.0)
#else
#define SHADOW_DIR_Z				(u_directionalLightDirection[0])
#define SHADOW_ORIGIN				vec3(0.0, 0.0, 0.0)
#endif

float sdwDepth2nld( in float depth )
{
	return ( depth -SHADOW_DEPTH_NEAR ) /SHADOW_DEPTH_RANGE;
}

float sdwNld2depth( in float nld )
{
	return nld *SHADOW_DEPTH_RANGE + SHADOW_DEPTH_NEAR;
}

	// ortho
vec3 sdwProject( in vec3 worldPos )
{
	vec3 viewPos = worldPos -SHADOW_ORIGIN;

	return vec3(
		dot( SHADOW_DIR_X, viewPos ) /SHADOW_PROJ_WORLD_RANGE *2.0,
		dot( SHADOW_DIR_Y, viewPos ) /SHADOW_PROJ_WORLD_RANGE *2.0,
		sdwDepth2nld( dot( SHADOW_DIR_Z, viewPos ) ) *2.0 -1.0
	);
}


