
#include "pass_colloid_common.finc"

///////////////////////////////////////////////////////////
// uniforms

uniform sampler2D u_texture;

///////////////////////////////////////////////////////////
// varyings

varying vec2 v_projCoord;
varying vec2 v_texCoord;
varying vec4 v_color;

///////////////////////////////////////////////////////////
// tech

#define FXAA_SUBPIX_SHIFT		(1.0/8.0)
#define FXAA_SPAN_MAX			(8.0)
#define FXAA_REDUCE_MUL			(1.0/8.0)
#define FXAA_REDUCE_MIN			(1.0/128.0)
#define FXAA_TEXUV_REDUCE		(0.8)

vec4 FXAA( in vec2 uv, in sampler2D tex, in vec3 rcpFrameZero )
{
	vec2 uvShifted = uv - ( rcpFrameZero.xy*( 0.5 +FXAA_SUBPIX_SHIFT ) );

	// fetch textures
	vec4 rgbNW = texture2D(tex, uvShifted);
	vec4 rgbNE = texture2D(tex, uvShifted + rcpFrameZero.xz);
	vec4 rgbSW = texture2D(tex, uvShifted + rcpFrameZero.yz);
	vec4 rgbSE = texture2D(tex, uvShifted + rcpFrameZero.xy);
	vec4 rgbM  = texture2D(tex, uv);

	// luminance proj
	float lumaNW = calcLuma(rgbNW.xyz);
	float lumaNE = calcLuma(rgbNE.xyz);
	float lumaSW = calcLuma(rgbSW.xyz);
	float lumaSE = calcLuma(rgbSE.xyz);
	float lumaM  = calcLuma(rgbM.xyz);

	// calc min max
	float lumaMin = min(lumaM, min(min(lumaNW, lumaNE), min(lumaSW, lumaSE)));
	float lumaMax = max(lumaM, max(max(lumaNW, lumaNE), max(lumaSW, lumaSE)));

	// discard
	//if ( ( lumaMax - lumaMin ) < 0.05 )
	//{ return vec4(rgbM.xyz, 1.0); }

	// calc edge direction
	vec2 dir; 
	dir.x = -((lumaNW + lumaNE) - (lumaSW + lumaSE));
	dir.y =  ((lumaNW + lumaSW) - (lumaNE + lumaSE));

	float dirReduce = max(
		(lumaNW + lumaNE + lumaSW + lumaSE) * (0.25 * FXAA_REDUCE_MUL),
		FXAA_REDUCE_MIN);
	float rcpDirMin = 1.0/(min(abs(dir.x), abs(dir.y)) + dirReduce);
	dir = min(vec2( FXAA_SPAN_MAX,  FXAA_SPAN_MAX), 
		  max(vec2(-FXAA_SPAN_MAX, -FXAA_SPAN_MAX), 
		  dir * rcpDirMin)) * rcpFrameZero.xy;

	// output
	vec4 c = vec4(0.0, 0.0, 0.0, 1.0);
	vec3 rgbA = (1.0/2.0) * (
		texture(tex, uv + dir * (1.0/3.0 - 0.5) *FXAA_TEXUV_REDUCE).xyz +
		texture(tex, uv + dir * (2.0/3.0 - 0.5) *FXAA_TEXUV_REDUCE).xyz);
	vec3 rgbB = rgbA * (1.0/2.0) + (1.0/4.0) * (
		texture(tex, uv + dir * (0.0/3.0 - 0.5) *FXAA_TEXUV_REDUCE).xyz +
		texture(tex, uv + dir * (3.0/3.0 - 0.5) *FXAA_TEXUV_REDUCE).xyz);
	float lumaB = calcLuma(rgbB);

	if( (lumaB < lumaMin) || (lumaB > lumaMax) )
	{
		c.rgb = rgbA;
	}
	else
	{
		c.rgb = rgbB;
	}
	return c;
}

///////////////////////////////////////////////////////////
// pass

void main()
{
	vec3 rcpFrameZero = vec3( 1.0 /SCREEN_WIDTH, -1.0 /SCREEN_HEIGHT, 0.0 );
	vec4 outColor = FXAA( v_texCoord, u_texture, rcpFrameZero );

	gl_FragColor = outColor;
}

