

/*
ORIGINAL CODE:
________________________________________________

The MIT License (MIT) Copyright (c) 2015 Jam3

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/


#include "pass_colloid_common.finc"

///////////////////////////////////////////////////////////
// uniforms

uniform sampler2D u_texture;
uniform vec4 u_frameResolution;
uniform vec4 u_blurDirection;

///////////////////////////////////////////////////////////
// varyings

varying vec2 v_projCoord;
varying vec2 v_texCoord;
varying vec4 v_color;

///////////////////////////////////////////////////////////
// tech

#ifndef GAUSS_STEP
#define GAUSS_STEP			(5)
#endif

vec4 gauss( in vec2 uv, in sampler2D tex, in vec2 rcpFrame, in vec2 direction )
{
	vec4 c = vec4(0.0);
	vec2 miuv = vec2(0.01), mauv = vec2(0.99);
#if (GAUSS_STEP == 1)
	c = texture(tex, uv);
#elif (GAUSS_STEP == 3)
	vec2 offset = 1.0 * direction * rcpFrame;
	c += texture(tex, uv + offset) * 0.5;
	c += texture(tex, uv - offset) * 0.5;
#elif (GAUSS_STEP == 5)
	vec2 offset = 1.33333334 * direction * rcpFrame;
	c += texture(tex, uv) * 0.294117647058;
	c += texture(tex, uv + offset) * 0.35294117647;
	c += texture(tex, uv - offset) * 0.35294117647;
#elif (GAUSS_STEP == 9)
	vec2 offset = 1.3846153846 * direction * rcpFrame;
	vec2 offsetFar = 3.2307692308 * direction * rcpFrame;
	c += texture(tex, uv) * 0.2270270270;
	c += texture(tex, clamp( uv + offset, miuv, mauv )) * 0.3162162162;
	c += texture(tex, clamp( uv - offset, miuv, mauv )) * 0.3162162162;
	c += texture(tex, clamp( uv + offsetFar, miuv, mauv )) * 0.0702702703;
	c += texture(tex, clamp( uv - offsetFar, miuv, mauv )) * 0.0702702703;
#elif (GAUSS_STEP == 13)
	vec2 offset = 1.41176470 * direction * rcpFrame;
	vec2 offsetMiddle = 3.29411764 * direction * rcpFrame;
	vec2 offsetFar = 5.17647058 * direction * rcpFrame;
	c += texture(tex, uv) * 0.1964825501;
	c += texture(tex, clamp( uv + offset, miuv, mauv )) * 0.2969069646;
	c += texture(tex, clamp( uv - offset, miuv, mauv )) * 0.2969069646;
	c += texture(tex, clamp( uv + offsetMiddle, miuv, mauv )) * 0.09447039785;
	c += texture(tex, clamp( uv - offsetMiddle, miuv, mauv )) * 0.09447039785;
	c += texture(tex, clamp( uv + offsetFar, miuv, mauv )) * 0.01038136240;
	c += texture(tex, clamp( uv - offsetFar, miuv, mauv )) * 0.01038136240;
#endif
	return c;
}

///////////////////////////////////////////////////////////
// pass

void main()
{
	vec4 outColor = gauss( v_texCoord, u_texture, u_frameResolution.zw, u_blurDirection.xy );
	gl_FragColor = outColor;
}

