
#include "pass_colloid_common.finc"

///////////////////////////////////////////////////////////
// uniforms

uniform sampler2D u_texture;

// color grading
uniform vec4 u_colorGradingParamR;
uniform vec4 u_colorGradingParamG;
uniform vec4 u_colorGradingParamB;
uniform float u_colorGradingMixAlpha;

// vignette
uniform vec4 u_vignetteParam;

///////////////////////////////////////////////////////////
// varyings

varying vec2 v_projCoord;
varying vec2 v_texCoord;
varying vec4 v_color;

///////////////////////////////////////////////////////////
// pass

float applyCG( float channel, vec4 param )
{
	float th = param.x;
	float xscale = param.y;
	float rate = param.z;

	float theta = clamp( ( channel -th ) *xscale *2.0, -1.0, 1.0 ) *3.14159265358979 *0.5;
	float curve = sin( theta ) *0.5 +0.5;

	return clamp( mix( channel, curve, rate ), 0.0, 1.0 );
}

void main()
{
	vec4 color = texture( u_texture, v_texCoord );
	vec4 outColor = color;

#ifndef DISABLE_COLOR_GRADING
	{
		vec4 resolved = vec4( applyCG(color.r, u_colorGradingParamR), applyCG(color.g, u_colorGradingParamG), applyCG(color.b, u_colorGradingParamB), outColor.a );
		outColor.rgb = mix( outColor.rgb, resolved.rgb, u_colorGradingMixAlpha );
	}
#endif

#ifndef DISABLE_VIGNETTE
	{
		float len = distance( v_texCoord.xy, vec2(0.5, 0.5) );
		float resolved = mix( 1.0, (1.0 -u_vignetteParam.z), clamp( ( len -u_vignetteParam.x ) *u_vignetteParam.y, 0.0, 1.0 ) );
		outColor.rgb *= resolved;
	}
#endif

	gl_FragColor = outColor;
}

