
#include "pass_colloid_common.finc"

///////////////////////////////////////////////////////////
// uniforms

uniform sampler2D u_texture;

///////////////////////////////////////////////////////////
// varyings

varying vec2 v_projCoord;
varying vec2 v_texCoord;
varying vec4 v_color;

///////////////////////////////////////////////////////////
// tech

#define SAMPLE_NUM				(6)
#define EDGE_DISTANCE			(uDepthParam.w)

uniform vec4 uDepthParam;
uniform vec4 uSampleParam;
uniform vec4 uSample01;
uniform vec4 uSample23;
uniform vec4 uSample45;

float perSample( in vec2 uv, in sampler2D tex, vec3 rcpFrameZero, in vec2 sample, in float cdepth )
{
	float depthFactor = uDepthParam.x;

	vec2 uvDiff = rcpFrameZero.xy * sample;
	float ldepth = decompress32( texture(tex, uv + uvDiff).xyzw );
	float rdepth = decompress32( texture(tex, uv - uvDiff).xyzw );

	float ld = ( ldepth - cdepth );
	float rd = ( rdepth - cdepth );

	float ao = 0.0;
#if 0
		// angle base
	vec3 lv = vec3( sample, ld *depthFactor );
	vec3 rv = vec3( sample, rd *depthFactor );

	#if 0
	ao += step( ld, EDGE_DISTANCE ) * ( 1.0 - max( normalize(lv).z, 0.0 ) );
	ao += step( rd, EDGE_DISTANCE ) * ( 1.0 - max( normalize(rv).z, 0.0 ) );
	#else
	ao += step( ld, EDGE_DISTANCE ) * step( rd, EDGE_DISTANCE ) * ( 1.0 - max( normalize(lv).z + normalize(rv).z, 0.0 ) *0.5 );
	#endif
#elif 0
		// depth point base
	ao += step( 0.0, ld ) * clamp( 1.0 -( ld /EDGE_DISTANCE ), 0.0, 1.0 );
	ao += step( 0.0, rd ) * clamp( 1.0 -( rd /EDGE_DISTANCE ), 0.0, 1.0 );
#else
		// depth volume base
	ao += clamp( ld /EDGE_DISTANCE, -1.0, 1.0 ) * clamp( 1.0 -( -ld -EDGE_DISTANCE )/EDGE_DISTANCE, 0.0, 1.0 );
	ao += clamp( rd /EDGE_DISTANCE, -1.0, 1.0 ) * clamp( 1.0 -( -rd -EDGE_DISTANCE )/EDGE_DISTANCE, 0.0, 1.0 );
	ao = ( ao *0.5 + 1.0 );// normalized in total
#endif
	return ao;
}

float SSAO( in vec2 uv, in sampler2D tex, in vec3 rcpFrameZero )
{
	//uv += vec2(-rcpFrameZero.x, rcpFrameZero.y);// 1pixel

	float centerDepth = decompress32( texture(tex, uv).xyzw );
	float sampleMul = max( mix( uSampleParam.x, uSampleParam.y, centerDepth ), 0.0 );

	float ao = 0.0;
	ao += perSample( uv, tex, rcpFrameZero, uSample01.xy *sampleMul, centerDepth );
	ao += perSample( uv, tex, rcpFrameZero, uSample01.zw *sampleMul, centerDepth );
	ao += perSample( uv, tex, rcpFrameZero, uSample23.xy *sampleMul, centerDepth );
	ao += perSample( uv, tex, rcpFrameZero, uSample23.zw *sampleMul, centerDepth );
	ao += perSample( uv, tex, rcpFrameZero, uSample45.xy *sampleMul, centerDepth );
	ao += perSample( uv, tex, rcpFrameZero, uSample45.zw *sampleMul, centerDepth );

	// half
	ao = clamp( (ao /float(SAMPLE_NUM)), 0.0, 1.0 );

	// dist atten
	ao = mix( ao, 1.0, clamp( ( centerDepth -uDepthParam.y ) /uDepthParam.z, 0.0, 1.0 ) );

	return ao;
}

///////////////////////////////////////////////////////////
// pass

void main()
{
	vec3 rcpFrameZero = vec3( 1.0 /SCREEN_WIDTH, -1.0 /SCREEN_HEIGHT, 0.0 );
	float ao = SSAO( v_texCoord, u_texture, rcpFrameZero );

#if 0
	ao = decompress32( texture(u_texture, v_texCoord).xyzw ) *10.0;
#endif

	//gl_FragColor = vec4( ao, ao, ao, ao );
	gl_FragColor = vec4( 0.0, 0.0, 0.0, 1.0 -ao );
}

